class Metrics:
    USERS = "ga:users"
    NEWUSERS = "ga:newUsers"
    SESSIONS = "ga:sessions"
    PAGEVIEWS = "ga:pageviews"
    BOUNCERATE = "ga:bounceRate"
    GOAL_1 = "ga:goal1ConversionRate"
    GOAL_2 = "ga:goal2ConversionRate"
    GOAL_3 = "ga:goal3ConversionRate"


class Dimensions:
    DEVICECATEGORY = "ga:deviceCategory"
    NEWUSERS = "ga:newUsers"
    SESSIONS = "ga:sessions"
    PAGEVIEWS = "ga:pageviews"
